﻿using System.Runtime.InteropServices;

using u8 = System.Byte;
using u16 = System.UInt16;
using u32 = System.UInt32;
using s32 = System.Int32;

namespace ActisenseComms
{
    namespace API
    {
        public partial class Const 
        {
            public const int ddMAXDATAu8_n = 223;
            public const int ddMaxPGNListSize = 55;
        };
        /*************************************************************************
	        Definitions of ports
        **************************************************************************/
        public enum NumPorts_t : int {
	        /* Number of baud codes for an NGW */
	        ddNGWNUMPORTS = 2,
	        /* Number of baud codes for an NGT */
	        ddNGTNUMPORTS = 2,
	        /* Maximum number of communication ports for any Actisense product-
	           used to set array size for data information coming back from
	           products - this figure could change depending upon future Actisense
	           product requirements. */
	        ddMAXNUMPORTS = 16
        };

        public enum DecodeReason_t {
	        /* Data arrived ok, data can be retrieved from the API buffer */
	        drDecodeDataArrived,
	        /* Data did not arrive back for the command sent - a timeout occurred
               - the data type that timed out will be flagged */
	        drDecodeTimeout,
	        /* Data invalid - data was invalidated, probably because the user code
               cancelled a callback while the callback was being actioned, causing
               the data buffer for that callback to be destroyed before the user
               callback code could access it */
	        drDecodeDataInvalid,
        };

         /* ----------------------------------------------------------------------
            sDecodeHardwareInfo	
            ACommsCommand_GetHardwareInfo retuns the Hardware information.
            ---------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeHardwareInfo {
            public sDecodeTag Tag;
            public u32 BootloaderSoftwareVersion;	/* Bootloader program's software version  */
            public sExcelDate BootloaderDate;		/* Bootloader program's "Date of program" */
            public sExcelTime BootloaderTime;		/* Bootloader program's "Time of program" */
            public u32 FirmwareSoftwareVersion;	    /* Main program's firmware version number */
            public sExcelDate FirmwareDate;		    /* Main program's firmware "Date of program" */
            public sExcelTime FirmwareTime;		    /* Main program's firmware "Time of program" */
            public u32 HardwareVersion;			    /* Main program's hardware version number */
            public u32 TotalOperatingTime;			/* Total running time in seconds */
            public sExcelTime TotalOperatingTimeX;	/* Total running time in Excel Time format */
            public u32 ModelSubID;					/* Model Sub ID number */
            public u32 ModeID;						/* Mode ID number */
        } ;
        
        /* -----------------------------------------------------------------------
           sDecodeOperatingMode
           ACommsCommand_GetOperatingMode returns a single 'Operating Mode' ID that
           indicates what 'Rule set' the target is currently using to operate.

           ddOperatingMode
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeOperatingMode {
            public sDecodeTag Tag;
            public u32 ModeID;						/* Current Operating Mode. Unique ID 
									                   across all ARL models */
        };
        
        /* -----------------------------------------------------------------------
           sDecodePortBaudCodes	
           ACommsCommand_GetPortBaudCodes returns the port Baud Codes
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodePortBaudCodes {
            const int cMaxBaudCodeSize = (int) NumPorts_t.ddMAXNUMPORTS;
            public sDecodeTag Tag;
            public u32 NumBaudCodes;					    /* Actual number of Baud codes */
            public fixed u8 HardProtocol[cMaxBaudCodeSize];	/* Hardware-Protocol enum type */
            public fixed u8 BaudCode[cMaxBaudCodeSize];		/* Baud Code enum type */
        };
        
        /* -----------------------------------------------------------------------
           sDecodeArray_u8
           Generic Data array structure, used as a return type where arrays of
           unsigned 8-bit data are returned from a target device.

           ACommsDecode_GetPortBaudCodes
           ACommsDecode_GetPortPCodes
           ACommsDecode_GetPortDupDelete
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodeArray_u8 {
            public sDecodeTag Tag;
            public u32 nData;					        /* Number of valid data members actually stored in this structure */
            public fixed u8 Data[Const.ddMAXDATAu8_n];	/* Array where the data will be stored -  only the first nData members will be valid */
        };

        /* -----------------------------------------------------------------------
           sDecodeTotalTime
           Used for ACommsCommand_GetTotalTime returns one the total running time in
           seconds of the target device
           ------------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeTotalTime {
            public sDecodeTag Tag;
            public u32 Valid;			        /* Validity of the time value. Can only be:
						                           0 (for time invalid) or
						                           1 (time is valid) */
            public u32 Time;			        /* Current running time in seconds */
        }
        
        /* -----------------------------------------------------------------------
           sProductInfoN2K
           ACommsCommand_GetProductInfoN2K
           ddProductInfoN2K
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodeProductInfoN2K {
            const int ddMaxManStr_n = 33;
            public sDecodeTag Tag;
            public u16 Version;		    /* NMEA 2000 Version supported, contains 3 d.p. div by 1000 to get floating point number (x.xxx) */
            public u16 ProductCode;	    /* NMEA 2000 Manufacturers Product Code (0 to 65355) */
            public u8 CertLevel;		/* NMEA 2000 Certification Level type (A/0 or B/1) */
            public u8 LEN;			    /* Load Equivalency (current drain in multiples of 50 mA) */
            public fixed u8 ManModelIDStr[ddMaxManStr_n];         /* Manufacturer Model ID ASCII string */
            public fixed u8 ManSoftwareVersionStr[ddMaxManStr_n]; /* Manufacturer Software version ASCII string */
            public fixed u8 ManHardwareVersionStr[ddMaxManStr_n]; /* Manufacturer Model/Hardware version ASCII string */
            public fixed u8 ManSerialCodeStr[ddMaxManStr_n];      /* Manufacturer Model serial code ASCII string */
        };
        
        /* -----------------------------------------------------------------------
           sDecodeCanConfig
           ddCANConfig
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodeCanConfig {
            const int ddCanNameCount = 8;
            public sDecodeTag Tag;
            public u8 PreferredAddress;		         /* PreferredAddress (to claim at startup) */
            public fixed u8 CANName[ddCanNameCount]; /* 64-bit CAN Name (used for Address Claiming) */
            public u8 PreviousSrcAddress;		     /* Previous Address (claimed in last session) */
            public u8 CurrentSrcAddress;		     /* Current Source Address (0 - 251) */
            public u8 AddressClaimStatus;		     /* Source Address 'Claimed' status (YES or NO) */
        };
        
        /* -----------------------------------------------------------------------
           sDecodeCanInfoField
           ddCANInfoField1
           ddCANInfoField2
           ddCANInfoField3
           ----------------------------------------------------------------------- */
        /* Generic CAN Info field maximum string length (in characters) - this
           is the total maximum that can be returned by the device */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodeCanInfoField {
	        const int cCANInfoFieldMaxLen = 70;
            /* Indiviual actual available CAN Info field string lengths (in characters)
                for each info string that can be set by the user : */
            const int cCANInfo1FieldMaxLen = cCANInfoFieldMaxLen;
            const int cCANInfo2FieldMaxLen = cCANInfoFieldMaxLen;
            const int cCANInfo3FieldMaxLen = 38;
            public sDecodeTag Tag;
            public u8 Type;                   /* Type = 0 for ASCII, = 1 For Unicode */
            public u8 Size;                   /* Length of ASCII/Unicode text string in bytes */
	        /* Info String - sized to fit maximum length of CAN info string, plus
               an extra two bytes, which will always be set to NULL (0) */
            public fixed u8 String[cCANInfoFieldMaxLen + 2];
        };
        
        /* -----------------------------------------------------------------------
           sDecodeRxPGN
           ddEnableRxPGN
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeRxPGN {
            public sDecodeTag Tag;
            public u32 PGN;                    /* PGN ID (0 to 131071) */
            public PGNEnable_t Enable;         /* Enable (NO/0, YES/1 or Respond/2 */
            public u32 Mask;                   /* PGN Mask (PGNMasks_t) (0 to 2^32) */
        };
        
        /* -----------------------------------------------------------------------
           sDecodeTxPGN
           ddEnableTxPGN
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeTxPGN {
            public sDecodeTag Tag;
            public u32 PGN;                    /* PGN ID (0 to 131071) */
            public PGNEnable_t Enable;         /* Enable (NO/0, YES/1 or Respond/2 */
            public u32 Rate;                   /* Tx Rate */
            public u32 Timeout;                /* Tx Timeout */
            public u32 Priority;               /* Tx priority */
        };
        
        /* -----------------------------------------------------------------------
           sDecodeRxPGNList
           ddRxPGNEnableList
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodeRxPGNList {
            public sDecodeTag Tag;
            public u32 Size;
            public fixed u32 PGN[Const.ddMaxPGNListSize];  /* Rx PGN */
            public fixed u32 Mask[Const.ddMaxPGNListSize]; /* Rx Mask */
        };
        
        /* -----------------------------------------------------------------------
           sDecodeTxPGNList
           ddTxPGNEnableList
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodeTxPGNList {
            public sDecodeTag Tag;
            public u32 Size;
            public fixed u32 PGN[Const.ddMaxPGNListSize];
            public fixed u32 Rate[Const.ddMaxPGNListSize];      /* Tx Rate */
            public fixed u32 Timeout[Const.ddMaxPGNListSize];	/* Tx Timeout */
            public fixed u32 Priority[Const.ddMaxPGNListSize];	/* Tx priority */
        };
        
        /* -----------------------------------------------------------------------
	       PGN list status structure data types :
	       Returned by commands
		       ACommsCommand_ActivatePGNEnableLists and
		       ACommsCommand_GetParamsPGNEnableLists
	       Taken as arguments by the functions
		       ACommsDecode_GetActivatePGNEnableListResponse and
		       ACommsDecode_GetParamsPGNEnableLists
           ----------------------------------------------------------------------- */
        /* PGN Enable list status structure */
         [StructLayout(LayoutKind.Sequential)]
         public struct sDecodePGNEnableListStatus {
             public sDecodeTag Tag;
             public u16 RealRxPGNListSize;			/* Number of Real Rx PGNs active in List */
             public u16 RealRxPGNListCapacity;		/* Maximum Real Capacity of Rx List */
             public u16 VirtualRxPGNListSize;		/* Number of Virtual Rx PGNs active in Rx List */
             public u16 VirtualRxPGNListCapacity;	/* Maximum Virtual Capacity of Rx List */
             public u16 TxPGNListSize;				/* Number of Virtual Tx PGNs active in Tx List (All Tx PGNs are virtual) */
             public u16 TxPGNListCapacity;			/* Maximum Virtual Capacity of Tx List */
             public u8 ListHardwareSyncStatus;      /* List->Hardware synchronisation status (YES(1), NO(0)) */
        };
        
        /* -----------------------------------------------------------------------
           sDecodeStartupStatus
           Startup status structure data types - this message is never
           "Requested", it is always Broadcast every time the device powers on.
           ------------------------------------------------------------------------ */
        /* System status structure */
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeStartupStatus	{
            public sDecodeTag Tag;
            public u16 FirmwareVersion;		/* Firmware version (/1000 for fp number) */
            public u32 ResetStatus;			/* (cause of last reset) */
        };

        /* -----------------------------------------------------------------------
	       System status structure data types :
	       The main structure type is the sSystemStatus, the other structures
           are member structures of sSystemStatus
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeIndiPercent {
            public u8 RxBandwidth;
            public u8 RxLoad;
            public u8 RxFiltered;
            public u8 RxDropped;
            public u8 TxBandwidth;
            public u8 TxLoad;
        };
        [StructLayout(LayoutKind.Sequential)]
        public struct sDecodeUniPercent {
            public u8 Bandwidth;
            public u8 Deleted;
            public u8 BufferLoading;
            public u8 PointerLoading;
        };
        /* System status structure */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDecodeSystemStatus {
            /* Maximum number of Indi and Uni Buffers that can currently be handled */
	        const int APIDECODE_maxnIndi = 16;
            const int APIDECODE_maxnUni = 8;
            public sDecodeTag Tag;
            public u32 nIndi;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = APIDECODE_maxnIndi)]
            public sDecodeIndiPercent[] Indi;
            public u32 nUni;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = APIDECODE_maxnUni)]
            public sDecodeUniPercent[] Uni;
        };
        
        /* -----------------------------------------------------------------------
	       Time Profiler Debug status structure data types :
	       The main structure type is the sDbgTimeProfiler, the other structure
           is a member structure of sDbgTimeProfiler
           ----------------------------------------------------------------------- */
        [StructLayout(LayoutKind.Sequential)]
        public struct sTimeProfile {
            /* Values received in Time Profile status message */
            public u32 MaxDuration;
            public u32 AccumDuration;
            public u32 InstanceCount;
            /* Values derived from the Time Profile status message */
            public float Period;
            public float PercentDuration;
            public float AverageDuration;
        };
        /* Debug Time Profile status structure */
        [StructLayout(LayoutKind.Sequential)]
        public unsafe struct sDbgTimeProfiler {
            /* Maximum number of Time Profiles that can currently be handled */
            const int APIDECODE_maxnTProfile = 8;
            public sDecodeTag Tag;
            public u32 nProfile;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = APIDECODE_maxnTProfile)]
            public sTimeProfile[] Profile;
        };
    }
}
